// TidyScreen Module
// $Author: Jeffrey J Weimer$
// SVN History: $Revision: 1456 $ on $Date: 2010-09-26 18:28:23 -0700 (Sun, 26 Sep 2010) $
//
// See the Help file for further information

#pragma rtGlobals=1
#pragma IgorVersion=6.20
#pragma version=1.4

#pragma IndependentModule=TidyScreen

#include "ScreenSizer", version >= 2.5

//#define TEST		// uncomment this line to do development

Static StrConstant thePackage="TidyScreen"
//Static StrConstant thePackageFolder=""
Static StrConstant theProcedureFile = "TidyScreen.ipf"
Static StrConstant thePackageInfo = "A control panel to lay out windows in percentage values of screen size"
Static StrConstant thePackageAuthor = "Jeffrey J Weimer"
Static Constant thePackageVersion = 1.4
Static Constant hasHelp = 1
Static Constant removable = 1

Static Constant tswidth = 165
Static Constant tsheight = 125

// Package Constants

// Menu

Menu "Misc", hideable
	Submenu "Panels"
		"Tidy Screen", MakeTidyScreenPanel()
	end
end

// Setup

Function SetupTidyScreen()
	
	string theCmd
	sprintf theCmd, "ProcGlobal#PackageExists(\"%s\")", thePackage
	Execute/Q/Z theCmd
	NVAR V_exists
	if (NVAR_exists(V_exists))
		sprintf theCmd,"\"%s\"", thePackage
//		sprintf theCmd, "%s,folder=\"%s\"", theCmd, thePackageFolder
		sprintf theCmd, "%s,file=\"%s\"", theCmd, theProcedureFile
		sprintf theCmd, "%s,info=\"%s\"", theCmd, thePackageInfo
		sprintf theCmd "%s,author=\"%s\"", theCmd, thePackageAuthor
		sprintf theCmd, "%s,version=%f", theCmd, thePackageVersion
		sprintf theCmd, "%s,hasHelp=%d", theCmd, hasHelp
		switch(V_exists)
			case 0:
				sprintf theCmd, "ProcGlobal#PackageSetup(%s)", theCmd
				break
			case 1:
				sprintf theCmd, "ProcGlobal#PackageUpdate(%s)", theCmd
				break
		endswitch
		Execute/Q/Z theCmd
	endif
	return 0
end

// Tidy Screen Panel

Function MakeTidyScreenPanel()

	if (WinType("TidyScreen")==7)
		DoWindow/F TidyScreen
		return 0
	endif

	// put up the panel
	
	string GIMN = GetIndependentModuleName(), theCmd
	sprintf theCmd, "%s#ScreenSizer#Initialize(0)", GIMN
	Execute/Q/Z theCmd
	sprintf theCmd, "%s#ScreenSizer#MakePanel(60,25,%d,%d,\"TidyScreen\",other=\"/N=TidyScreen/K=1\")", GIMN,tswidth,tsheight
	Execute/Q/Z theCmd
#ifndef TEST
	ModifyPanel/W=TidyScreen noEdit=1,fixedSize=1
#endif

	// reset the panel to the Default desktop rather than the current desktop

	Execute/Z/Q "ACL_WindowDesktops#SetWindowDesktop(\"TidyScreen\",0)"

	// the NOT checkbox
	
	CheckBox not,pos={6,9},size={25,14},title="!"
	CheckBox not,help={"Check this to have the inverse applied to your selection of what windows to change"}
	CheckBox not,font="Courier",fSize=12,value= 0
	
	// what windows?
	
	PopupMenu What,pos={35,6},size={120,20},title=""
	PopupMenu What,help={"Select which kinds of windows to change"},font="Courier"
	PopupMenu What,fSize=12
	PopupMenu What,mode=1,popvalue="Graphs",value= #"\"Graphs;Tables;Layouts;Notebooks;\\M1(Panels;Procedures;XOPS;\""
	
	// front window selector
	
	CheckBox front,pos={10,27},size={54,14},title="Front",proc=DoFrontOnly
	CheckBox front,help={"Check this to operate only on the frontmost window"}
	CheckBox front,font="Courier",fSize=12,value= 0

	// margin
	
	SetVariable margin title="||",pos={109,52},size={45,20},limits={0,10,1},value= _NUM:0, proc=SetMargin
	
	// do it
	
	Button DoIt,pos={77,45},size={25,20},title="->", fColor=(16385,28398,65535),proc=DoIt
	Button DoIt,help={"Click this button to complete the task"},font="Courier"

	// procedure
	
	PopupMenu Task,pos={6,45},size={90,20},title="",proc=SelectTask
	PopupMenu Task,help={"Select the option to move or size windows"},font="Courier"
	PopupMenu Task,fSize=12,mode=1,popvalue="Move",value= #"\"Move;Size;SizeAR;Shift;\""

	// the version number
	
	sprintf theCmd, "v %g", thePackageVersion
	SetDrawLayer UserBack
	SetDrawEnv fsize= 9
	DrawText 110,40, theCmd
	DrawLine 4,71,157,71
	
	// Help
	
	Button GetHelp,pos={140,5},size={20,20},title="?",proc=GetHelp
	Button GetHelp,help={"Click this button to open the help file"},font="Courier"
	
	// remove me
	
	Button removeme,pos={140,28},size={20,20},title="X",proc=RemoveThis
	Button removeme,help={"Click this button to remove the TidyScreen package from this experiment"},font="Courier"
	
	// values
	
	SetVariable top,pos={12,80},size={70,15},title="V",help={"top size (%)"}
	SetVariable top,font="Arial",fSize=12,limits={0,100,5},value= _NUM:0
	SetVariable left,pos={12,98},size={70,15},title=">",help={"left size (%)"}
	SetVariable left,font="Arial",fSize=12,limits={0,100,5},value= _NUM:0
	SetVariable right,pos={84,80},size={70,15},title=">",help={"right size (%)"}
	SetVariable right,font="Arial",fSize=12,limits={0,100,5},value= _NUM:0
	SetVariable bottom,pos={84,98},size={70,15},title="V",help={"bottom size (%)"}
	SetVariable bottom,font="Arial",fSize=12,limits={0,100,5},value= _NUM:0

	SetupTidyScreen()
	
	return 0
	
End

Function SelectTask(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	switch( pa.eventCode )
		case 2:
			strswitch(pa.popStr)
				case "Move":
					SetVariable top, title="V",help={"top size (%)"}, limits={0,100,5},value= _NUM:0
					SetVariable left,title=">",help={"left size (%)"},limits={0,100,5},value= _NUM:0, disable = 0
					SetVariable right,title=">",font="Arial",help={"right size (%)"},limits={0,100,5}, disable=0 //,value= _NUM:0
					SetVariable bottom,title="V",font="Arial",help={"bottom size (%)"},limits={0,100,5},disable=0 //,value= _NUM:0
					ControlInfo What
					if (V_value==5)
						PopupMenu What,mode=1,popvalue="Graphs",value= #"\"Graphs;Tables;Layouts;Notebooks;\\M1(Panels;Procedures;XOPS;\""
					else
						PopupMenu What,mode=V_value,popvalue=S_value,value= #"\"Graphs;Tables;Layouts;Notebooks;\\M1(Panels;Procedures;XOPS;\""
					endif
					break
				case "Size":
					SetVariable top, title="|",help={"top size (%)"}, limits={0,100,5},value= _NUM:0
					SetVariable left,title="_",help={"left size (%)"},limits={0,100,5},value= _NUM:0, disable = 0
					SetVariable right, disable = 1
					SetVariable bottom, disable = 1
					ControlInfo What
					if (V_value==5)
						PopupMenu What,mode=1,popvalue="Graphs",value= #"\"Graphs;Tables;Layouts;Notebooks;\\M1(Panels;Procedures;XOPS;\""
					else
						PopupMenu What,mode=V_value,popvalue=S_value,value= #"\"Graphs;Tables;Layouts;Notebooks;\\M1(Panels;Procedures;XOPS;\""
					endif
					break
				case "SizeAR":
					SetVariable top, title="-/|",help={"top size (%)"}, limits={0,inf,0.1},value= _NUM:1
					SetVariable left, disable = 1
					SetVariable right, disable = 1
					SetVariable bottom, disable = 1
					ControlInfo What
					if (V_value==5)
						PopupMenu What,mode=1,popvalue="Graphs",value= #"\"Graphs;Tables;Layouts;Notebooks;\\M1(Panels;Procedures;XOPS;\""
					else
						PopupMenu What,mode=V_value,popvalue=S_value,value= #"\"Graphs;Tables;Layouts;Notebooks;\\M1(Panels;Procedures;XOPS;\""
					endif
					break
				case "Shift":
					ControlInfo front
					SetVariable top, title="V",help={"top size (%)"}, limits={-100,100,5} //,value= _NUM:0
					SetVariable left,title=">",help={"left size (%)"},limits={-100,100,5}, disable = 0 //,value= _NUM:0
					SetVariable right,title="D",font="Symbol",help={"left-to-right offset (%)"},limits={-100,100,5}, disable=(V_value)
					SetVariable bottom,title="D",font="Symbol",help={"top-to-bottom offset (%)"},limits={-100,100,5}, disable=(V_value)			
					ControlInfo What
					PopupMenu What,mode=V_value,popvalue=S_value,value= #"\"Graphs;Tables;Layouts;Notebooks;Panels;Procedures;XOPS;\""
					break
			endswitch
	endswitch

	return 0
End

Function DoFrontOnly(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba

	switch( cba.eventCode )
		case 2: // mouse up
			PopupMenu What, disable=(cba.checked)
			CheckBox not, disable=(cba.checked)
//			ControlInfo Task
//			strswitch(S_Value)
//				case "Move":
//				case "Size":
//				case "SizeAR":
//					SetVariable right,disable=0
//					SetVariable bottom,disable=0
//					SetVariable right,disable=(cba.checked)
//					SetVariable bottom,disable=(cba.checked)
//					break
//				case "Shift":
//					SetVariable right,disable=(cba.checked)
//					SetVariable bottom,disable=(cba.checked)
//					break
//			endswitch
			break
	endswitch

	return 0
End

Function DoIt(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			variable left, top, right, bottom, type, task, front
			string theKind="", theTask="", theCmd=""
			
			ControlInfo front
			if (V_value)
				front = 1
				theKind = ""
			else
				front = 0
				ControlInfo What
				theKind = S_value			
			endif

			ControlInfo Task
			theTask = S_value
			task = V_value
			if (!front)
				ControlInfo not
				if (V_value)
					strswitch(theKind)
						case "Graphs":
							type = (2+4+16+128+4096)
							type += (task==3)*64
							break
						case "Tables":
							type = (1+4+16+128+4096)
							type += (task==3)*64
							break
						case "Layouts":
							type = (1+2+16+128+4096)
							type += (task==3)*64
							break
						case "Notebooks":
							type = (1+2+4+128+4096)
							type += (task==3)*64
							break
						case "Panels":
							type = (1+2+4+16+128+4096)
							break
						case "Procedures":
							type = (1+2+4+16+4096)
							type += (task==3)*64
							break
						case "XOPs":
							type = (1+2+4+16+128)
							type += (task==3)*64
							break
					endswitch
				else
					strswitch(theKind)
						case "Graphs":
							type = 1
							break
						case "Tables":
							type = 2
							break
						case "Layouts":
							type = 4
							break
						case "Notebooks":
							type = 16
							break
						case "Panels":
							type = 64
							break
						case "Procedures":
							type = 128
							break
						case "XOPs":
							type = 4096
							break
					endswitch
				endif
			else
				theKind=WinName(1,(1+2+4+16+64+128+4096))
				type = WinType(theKind)
				switch(type)
					case 0: // procedure
						type = 128
						theKind = WinName(0,type)
						break
					case 7: // panel
						if (cmpstr(theTask,"Shift")!=0)
							return 0
						endif
					default:
						theKind = WinName(0,type)
						break
				endswitch
			endif

			ControlInfo left
			left = V_value
			ControlInfo right
			right = V_value
			ControlInfo top
			top = V_value
			ControlInfo bottom
			bottom = V_value
			
			string GIMN = GetIndependentModuleName()

			strswitch(theTask)
				case "Move":					
					if (front)
						sprintf theCmd, "%s#ScreenSizer#MoveAWindow(%d,%d,%d,%d,\"%s\")", GIMN,left,top,right,bottom,theKind
					else
						sprintf theCmd, "%s#ScreenSizer#MoveWindows(%d,%d,%d,%d,type=%d)", GIMN, left,top,right,bottom,type
					endif
					Execute/Q/Z theCmd					
					break
				case "Size":
					if (front)
						sprintf theCmd, "%s#ScreenSizer#SizeAWindow(%d,%d,\"%s\")", GIMN,left,top,theKind
					else
						sprintf theCmd, "%s#ScreenSizer#SizeWindows(%d,%d,type=%d)", GIMN, left,top,type
					endif
					Execute/Q/Z theCmd					
					break
				case "SizeAR":
					if (front)
						sprintf theCmd, "%s#ScreenSizer#SizeAWindowAR(%f,\"%s\")", GIMN,top,theKind
					else
						sprintf theCmd, "%s#ScreenSizer#SizeWindowsAR(%f,type=%d)", GIMN,top,type
					endif
					Execute/Q/Z theCmd					
					break
				case "Shift":
					if (front)
						sprintf theCmd, "%s#ScreenSizer#ShiftAWindow(%d,%d,\"%s\")", GIMN,left,top,theKind
					else
						sprintf theCmd, "%s#ScreenSizer#ShiftWindows(%d,%d,type=%d,dlr=%d,dtb=%d,exclude=\"TidyScreen;\")", GIMN, left,top,type,bottom,right
					endif
					Execute/Q/Z theCmd					
					break
			endswitch
			break
			return 0
			break
	endswitch

	return 0
End

Function SetMargin(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva

	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
		case 3: // Live update
			string GIMN = GetIndependentModuleName()
			string theCmd
			sprintf theCmd "%s#ScreenSizer#SetFrame(%d,%d,%d,%d)", GIMN,sva.dval, sva.dval,(100-sva.dval),(100-sva.dval)
			Execute/Q/Z theCmd
			break
	endswitch

	return 0
End

Function GetHelp(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			string theCmd 
			sprintf theCmd, "PackageHelp(\"%s\")", thePackage
			Execute/P/Q/Z theCmd
			break
	endswitch

	return 0
End

Function RemoveThis(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			KillWindow TidyScreen
			string theCmd 
			sprintf theCmd, "PackageRemove(\"%s\")", thePackage
			Execute/P/Q/Z theCmd
			break
	endswitch

	return 0
End
